@@ -104,6 +104,7 @@ class MPState(object):
         self.map = None
         self.map_functions = {}
         self.vehicle_type = None
+        self.vehicle_name = None
         self.settings = mp_settings.MPSettings(
             [ ('link', int, 1),
               ('altreadout', int, 10),
@@ -122,7 +123,6 @@ class MPState(object):
               ('shownoise', int, 1),
               ('basealt', int, 0),
               ('wpalt', int, 100),
-              ('parambatch', int, 10),
               ('requireexit', int, 0)]
             )
 
@@ -1078,11 +1078,14 @@ if __name__ == '__main__':
     from optparse import OptionParser
     parser = OptionParser("mavproxy.py [options]")
 
-    parser.add_option("--master",dest="master", action='append', help="MAVLink master port", default=[])
+    parser.add_option("--master", dest="master", action='append',
+                      metavar="DEVICE[,BAUD]", help="MAVLink master port and optional baud rate",
+                      default=[])
+    parser.add_option("--out", dest="output", action='append',
+                      metavar="DEVICE[,BAUD]", help="MAVLink output port and optional baud rate",
+                      default=[])
     parser.add_option("--baudrate", dest="baudrate", type='int',
-                      help="master port baud rate", default=115200)
-    parser.add_option("--out",   dest="output", help="MAVLink output port",
-                      action='append', default=[])
+                      help="default serial baud rate", default=115200)
     parser.add_option("--sitl", dest="sitl",  default=None, help="SITL output port")
     parser.add_option("--streamrate",dest="streamrate", default=4, type='int',
                       help="MAVLink stream rate")
@@ -1168,7 +1171,12 @@ Auto-detected serial ports are:
 
     # open master link
     for mdev in opts.master:
-        m = mavutil.mavlink_connection(mdev, autoreconnect=True, baud=opts.baudrate)
+        if ',' in mdev and not os.path.exists(mdev):
+            port, baud = mdev.split(',')
+        else:
+            port, baud = mdev, opts.baudrate
+
+        m = mavutil.mavlink_connection(port, autoreconnect=True, baud=int(baud))
         m.mav.set_callback(master_callback, m)
         if hasattr(m.mav, 'set_send_callback'):
             m.mav.set_send_callback(master_send_callback, m)
@@ -1188,7 +1196,12 @@ Auto-detected serial ports are:
 
     # open any mavlink UDP ports
     for p in opts.output:
-        mpstate.mav_outputs.append(mavutil.mavlink_connection(p, baud=opts.baudrate, input=False))
+        if ',' in p and not os.path.exists(p):
+            port, baud = p.split(',')            
+        else:
+            port, baud = p, opts.baudrate
+
+        mpstate.mav_outputs.append(mavutil.mavlink_connection(port, baud=int(baud), input=False))
 
     if opts.sitl:
         mpstate.sitl_output = mavutil.mavudp(opts.sitl, input=False)
